/**********************************************************************
 * Software Copyright Licensing Disclaimer
 *
 * This software module was originally developed by contributors to the
 * course of the development of ISO/IEC 14496-10 for reference purposes
 * and its performance may not have been optimized.  This software
 * module is an implementation of one or more tools as specified by
 * ISO/IEC 14496-10.  ISO/IEC gives users free license to this software
 * module or modifications thereof. Those intending to use this software
 * module in products are advised that its use may infringe existing
 * patents.  ISO/IEC have no liability for use of this software module
 * or modifications thereof.  The original contributors retain full
 * rights to modify and use the code for their own purposes, and to
 * assign or donate the code to third parties.
 *
 * This copyright notice must be included in all copies or derivative
 * works.  Copyright (c) ISO/IEC 2004-2016.
 **********************************************************************/

#ifndef __ALC_H__
#define __ALC_H__

#include "defines.h"
#include "global.h"

#if EXT3D

#ifndef CLIP
#define CLIP(x,min,max) ( (x)<(min)?(min):((x)>(max)?(max):(x)) )
#endif

typedef imgpel XPel;

//thickness of template. Is expressed in pixels
#define MAX_TEMPLATE_SIZE                                       (1)

// max MB Size 16
#define MAX_TEMPLATE_VOLUME                                     ((MAX_TEMPLATE_SIZE * 16) * 2 + (MAX_TEMPLATE_SIZE * MAX_TEMPLATE_SIZE))

//Threshold of difference between template values in reference and decoded frames
#define REF_DEC_LUMA_DIF_THRESHOLD                (30)          // if REF_DEC_LUMA_DIF_THRESHOLD == 256, then no comparison between ref and dec to be applied at all

//  EARLY_ALC_OFF_DETECTION -this define allows to compare sums of templates and exclude ALC_Correction if templates differencies are not crucial
//  EARLY_ALC_OFF_DETECTION_THRESHOLD defines how strong ref_sum/dec_sum will be quantizaed before comparison; it uses as follows:
//  ref_sum >> = EARLY_ALC_OFF_DETECTION_THRESHOLD;
//  dec_sum >> = EARLY_ALC_OFF_DETECTION_THRESHOLD;
#define EARLY_ALC_OFF_DETECTION                                 (1)
#define EARLY_ALC_OFF_DETECTION_THRESHOLD                       (4)

// fixed point macroses
#define SINGL_MULT_FACT_PREC                                    (15)
#define MULT_FACTOR_X_255                                       (255 << SINGL_MULT_FACT_PREC)

#define ALCOK                                                   (0)

#define TM_EMPTY                                                (0)
#define TM_FILLED                                               (1)

void ALC_Create();
void ALC_Destroy();
int  ALC_GetTM(  XPel **ppRef, int RefTM_startY, int RefTM_startX, 
                XPel **ppDec, int DecTM_startY, int DecTM_startX, int Blk_height, int Blk_width);
Boolean IsTMFilled();
void    ALC_TurnOFF();
XPel**  ALC_RetTM_buf();
int**   ALC_RetTM_tempbuf();
void    PiLC_TMPelCorHist_2DArray( XPel **ppOut, int   XOut,                              int h, int w );
void    PiLC_TMPelCorHist_1DArray( XPel  *pIn,   XPel *pOut, int StrideIn, int StrideOut, int h, int w );

#endif

#endif //__ALC_H__
